/**
* gpio macros for STM32. 
*
*  Copyright (c) 2009 Anton Gusev aka AHTOXA (HTTP://AHTOXA.NET)
*
*  Inspired by AVR macros from Askold Volkov 
*
* usage:
* #define PIN1 A, 5, H // PA5, active level = high
* #define PIN2 B, 12, L // PB12, active level = low
* 
* pmode(PIN1, PIN_CFG_OPP_50MHZ);  // configure PIN1 as output push-pull, 50MHz 
* pmode(PIN2, PIN_CFG_OPP_50MHZ);  // configure PIN2 as output push-pull, 50MHz 
* pmode(PIN3, PIN_CFG_IF);         // configure PIN3 as input floating
* on(PIN1);                        // switch PIN1 to active state (H)
* on(PIN2);                        // switch PIN2 to active state (L)
* off(PIN1);                       // switch PIN1 to inactive state (L)
* cpl(PIN1);                       // invert PIN1
*
* if (signal(PIN3))                // returns non-zero if pin input = active state (H for PIN1)
*
* if (latch(PIN1))                 // returns non-zero if pin output = active state (H for PIN1)
*
*/

#include "stm32f10x.h"
#include <stddef.h>

#define		MAKE_PIN_CFG(MODE, CNF)	( (MODE) | (CNF)<<2 )

#define		PIN_CFG_AI			MAKE_PIN_CFG ( 0, 0 )
#define		PIN_CFG_IF			MAKE_PIN_CFG ( 0, 1 )
#define		PIN_CFG_IP			MAKE_PIN_CFG ( 0, 2 )

#define		PIN_CFG_OPP_10MHZ	MAKE_PIN_CFG ( 1, 0 )
#define		PIN_CFG_OOD_10MHZ	MAKE_PIN_CFG ( 1, 1 )
#define		PIN_CFG_APP_10MHZ	MAKE_PIN_CFG ( 1, 2 )
#define		PIN_CFG_AOD_10MHZ	MAKE_PIN_CFG ( 1, 3 )

#define		PIN_CFG_OPP_2MHZ	MAKE_PIN_CFG ( 2, 0 )
#define		PIN_CFG_OOD_2MHZ	MAKE_PIN_CFG ( 2, 1 )
#define		PIN_CFG_APP_2MHZ	MAKE_PIN_CFG ( 2, 2 )
#define		PIN_CFG_AOD_2MHZ	MAKE_PIN_CFG ( 2, 3 )

#define		PIN_CFG_OPP_50MHZ	MAKE_PIN_CFG ( 3, 0 )
#define		PIN_CFG_OOD_50MHZ	MAKE_PIN_CFG ( 3, 1 )
#define		PIN_CFG_APP_50MHZ	MAKE_PIN_CFG ( 3, 2 )
#define		PIN_CFG_AOD_50MHZ	MAKE_PIN_CFG ( 3, 3 )

/**
* 
* pin mode: CNF[3:2] MODE[1:0]
* MODE:	00 - input
*		01 - output 10 MHz
*		10 - output 2 MHz
*		11 - output 50 MHz
* CNF:
*	in input mode:
*		00 - analog input
*		01 - input floating
*		10 - input pulled (PU when ODR=1, PD when ODR=0)
*	in output modes:
*		00 - push-pull
*		01 - open drain
*		10 - alternate push-pull
*		11 - alternate open drain
*
*/

#define		_setmodeHi(mode,port,pin)	\
				GPIO##port->CRH = (GPIO##port->CRH & (~(0x0F << ((pin-8)<<2)))) | (mode << ((pin-8)<<2))

#define		_setmodeLo(mode,port,pin)	\
				GPIO##port->CRL = (GPIO##port->CRL & (~((uint32_t)0x0F << (pin<<2)))) | ((uint32_t)mode << (pin<<2))

#define		_setmode0(mode,port,pin) _setmodeLo(mode, port, pin)
#define		_setmode1(mode,port,pin) _setmodeLo(mode, port, pin)
#define		_setmode2(mode,port,pin) _setmodeLo(mode, port, pin)
#define		_setmode3(mode,port,pin) _setmodeLo(mode, port, pin)
#define		_setmode4(mode,port,pin) _setmodeLo(mode, port, pin)
#define		_setmode5(mode,port,pin) _setmodeLo(mode, port, pin)
#define		_setmode6(mode,port,pin) _setmodeLo(mode, port, pin)
#define		_setmode7(mode,port,pin) _setmodeLo(mode, port, pin)

#define		_setmode8(mode,port,pin) _setmodeHi(mode, port, pin)
#define		_setmode9(mode,port,pin) _setmodeHi(mode, port, pin)
#define		_setmode10(mode,port,pin) _setmodeHi(mode, port, pin)
#define		_setmode11(mode,port,pin) _setmodeHi(mode, port, pin)
#define		_setmode12(mode,port,pin) _setmodeHi(mode, port, pin)
#define		_setmode13(mode,port,pin) _setmodeHi(mode, port, pin)
#define		_setmode14(mode,port,pin) _setmodeHi(mode, port, pin)
#define		_setmode15(mode,port,pin) _setmodeHi(mode, port, pin)

#define		_pmode(port, pin, level, mode)	_setmode##pin(mode,port,pin)
#define		pmode(pin, mode) _pmode(pin, mode)

#define		_make_onL(port, pin)	GPIO##port->BRR = (1UL<<pin)
#define		_make_onH(port, pin)	GPIO##port->BSRR = (1UL<<pin)
#define		_on(port, pin, level) _make_on##level(port, pin)
#define		on(pin) _on(pin)

#define		_make_offH(port, pin)	GPIO##port->BRR = (1UL<<pin)
#define		_make_offL(port, pin)	GPIO##port->BSRR = (1UL<<pin)
#define		_off(port, pin, level) _make_off##level(port, pin)
#define		off(pin) _off(pin)

// IDR bit-band address
#define IDR_BB_ADDR(port, pin)  PERIPH_BB_BASE + (GPIO##port##_BASE + offsetof(GPIO_TypeDef, IDR) - PERIPH_BASE) * 32 + pin * 4
// ODR bit-band address
#define ODR_BB_ADDR(port, pin)  PERIPH_BB_BASE + (GPIO##port##_BASE + offsetof(GPIO_TypeDef, ODR) - PERIPH_BASE) * 32 + pin * 4

#define		_checkH(port, pin)	(*(volatile uint32_t *)(IDR_BB_ADDR(port, pin)))
#define		_checkL(port, pin)	(!*(volatile uint32_t *)(IDR_BB_ADDR(port, pin)))
#define		_signal(port, pin, level) _check##level(port, pin)
#define		signal(pin) _signal(pin)

#define		_testH(port, pin)	(*(volatile uint32_t *)(ODR_BB_ADDR(port, pin)))
#define		_testL(port, pin)	(!*(volatile uint32_t *)(ODR_BB_ADDR(port, pin)))
#define		_latch(port, pin, level) _test##level(port, pin)
#define		latch(pin) _latch(pin)

#define		_cpl(port, pin, level) *(volatile uint32_t *)(ODR_BB_ADDR(port, pin)) = ~*(volatile uint32_t *)(ODR_BB_ADDR(port, pin));
#define		cpl(pin) _cpl(pin)


